@@ -12,8 +12,10 @@
 //! [#285]: https://gitlab.torproject.org/tpo/core/arti/-/issues/285
 
 use derive_builder::Builder;
+use derive_more::AsRef;
 use serde::Deserialize;
 use std::collections::HashMap;
+use std::convert::TryInto;
 use std::path::Path;
 use std::path::PathBuf;
 use std::time::Duration;
@@ -22,17 +24,17 @@ pub use tor_config::{CfgPath, ConfigBuildError, Reconfigure};
 /// Types for configuring how Tor circuits are built.
 pub mod circ {
     pub use tor_circmgr::{
-        CircMgrConfig, CircMgrConfigBuilder, CircuitTiming, CircuitTimingBuilder, PathConfig,
-        PathConfigBuilder, PreemptiveCircuitConfig, PreemptiveCircuitConfigBuilder,
+        CircMgrConfig, CircuitTiming, CircuitTimingBuilder, PathConfig, PathConfigBuilder,
+        PreemptiveCircuitConfig, PreemptiveCircuitConfigBuilder,
     };
 }
 
 /// Types for configuring how Tor accesses its directory information.
 pub mod dir {
     pub use tor_dirmgr::{
-        Authority, AuthorityBuilder, DirMgrConfig, DirMgrConfigBuilder, DownloadSchedule,
-        DownloadScheduleConfig, DownloadScheduleConfigBuilder, FallbackDir, FallbackDirBuilder,
-        NetworkConfig, NetworkConfigBuilder,
+        Authority, AuthorityBuilder, DirMgrConfig, DownloadSchedule, DownloadScheduleConfig,
+        DownloadScheduleConfigBuilder, FallbackDir, FallbackDirBuilder, NetworkConfig,
+        NetworkConfigBuilder,
     };
 }
 
@@ -46,6 +48,7 @@ pub mod dir {
 /// and requests.
 #[derive(Debug, Clone, Builder, Deserialize, Eq, PartialEq)]
 #[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(derive(Deserialize))]
 #[serde(deny_unknown_fields)]
 pub struct ClientAddrConfig {
     /// Should we allow attempts to make Tor connections to local addresses?
@@ -67,6 +70,7 @@ pub struct ClientAddrConfig {
 /// and requests—even those that are currently waiting.
 #[derive(Debug, Clone, Builder, Deserialize, Eq, PartialEq)]
 #[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(derive(Deserialize))]
 #[serde(deny_unknown_fields)]
 #[non_exhaustive]
 pub struct StreamTimeoutConfig {
@@ -74,17 +78,20 @@ pub struct StreamTimeoutConfig {
     /// to a host?
     #[builder(default = "default_connect_timeout()")]
     #[serde(with = "humantime_serde", default = "default_connect_timeout")]
+    #[builder_field_attr(serde(with = "humantime_serde::option"))]
     pub(crate) connect_timeout: Duration,
 
     /// How long should we wait before timing out when resolving a DNS record?
     #[builder(default = "default_dns_resolve_timeout()")]
     #[serde(with = "humantime_serde", default = "default_dns_resolve_timeout")]
+    #[builder_field_attr(serde(with = "humantime_serde::option"))]
     pub(crate) resolve_timeout: Duration,
 
     /// How long should we wait before timing out when resolving a DNS
     /// PTR record?
     #[builder(default = "default_dns_resolve_ptr_timeout()")]
     #[serde(with = "humantime_serde", default = "default_dns_resolve_ptr_timeout")]
+    #[builder_field_attr(serde(with = "humantime_serde::option"))]
     pub(crate) resolve_ptr_timeout: Duration,
 }
 
@@ -98,14 +105,6 @@ impl Default for ClientAddrConfig {
     }
 }
 
-impl From<ClientAddrConfig> for ClientAddrConfigBuilder {
-    fn from(cfg: ClientAddrConfig) -> ClientAddrConfigBuilder {
-        let mut builder = ClientAddrConfigBuilder::default();
-        builder.allow_local_addrs(cfg.allow_local_addrs);
-        builder
-    }
-}
-
 impl ClientAddrConfig {
     /// Return a new [`ClientAddrConfigBuilder`].
     pub fn builder() -> ClientAddrConfigBuilder {
@@ -120,18 +119,6 @@ impl Default for StreamTimeoutConfig {
     }
 }
 
-impl From<StreamTimeoutConfig> for StreamTimeoutConfigBuilder {
-    fn from(cfg: StreamTimeoutConfig) -> StreamTimeoutConfigBuilder {
-        let mut builder = StreamTimeoutConfigBuilder::default();
-        builder
-            .connect_timeout(cfg.connect_timeout)
-            .resolve_timeout(cfg.resolve_timeout)
-            .resolve_ptr_timeout(cfg.resolve_ptr_timeout);
-
-        builder
-    }
-}
-
 impl StreamTimeoutConfig {
     /// Return a new [`StreamTimeoutConfigBuilder`].
     pub fn builder() -> StreamTimeoutConfigBuilder {
@@ -172,6 +159,7 @@ fn default_dns_resolve_ptr_timeout() -> Duration {
 #[derive(Deserialize, Debug, Clone, Builder, Eq, PartialEq)]
 #[serde(deny_unknown_fields)]
 #[builder(build_fn(error = "ConfigBuildError"))]
+#[builder(derive(Deserialize))]
 pub struct StorageConfig {
     /// Location on disk for cached directory information.
     #[builder(setter(into), default = "default_cache_dir()")]
@@ -225,54 +213,6 @@ impl StorageConfig {
     }
 }
 
-impl From<StorageConfig> for StorageConfigBuilder {
-    fn from(cfg: StorageConfig) -> StorageConfigBuilder {
-        let mut builder = StorageConfigBuilder::default();
-        builder.state_dir(cfg.state_dir).cache_dir(cfg.cache_dir);
-        builder
-    }
-}
-
-/// Configuration for system resources used by Tor.
-///
-/// You cannot change this section on a running Arti client.
-#[derive(Deserialize, Debug, Clone, Builder, Eq, PartialEq)]
-#[serde(deny_unknown_fields)]
-#[builder(build_fn(error = "ConfigBuildError"))]
-#[non_exhaustive]
-pub struct SystemConfig {
-    /// Maximum number of file descriptors we should launch with
-    #[builder(setter(into), default = "default_max_files()")]
-    #[serde(default = "default_max_files")]
-    pub max_files: u64,
-}
-
-/// Return the default maximum number of file descriptors to launch with.
-fn default_max_files() -> u64 {
-    16384
-}
-
-impl Default for SystemConfig {
-    fn default() -> Self {
-        Self::builder().build().expect("Default builder failed")
-    }
-}
-
-impl SystemConfig {
-    /// Return a new SystemConfigBuilder.
-    pub fn builder() -> SystemConfigBuilder {
-        SystemConfigBuilder::default()
-    }
-}
-
-impl From<SystemConfig> for SystemConfigBuilder {
-    fn from(cfg: SystemConfig) -> SystemConfigBuilder {
-        let mut builder = SystemConfigBuilder::default();
-        builder.max_files(cfg.max_files);
-        builder
-    }
-}
-
 /// A configuration used to bootstrap a [`TorClient`](crate::TorClient).
 ///
 /// In order to connect to the Tor network, Arti needs to know a few
@@ -299,7 +239,7 @@ impl From<SystemConfig> for SystemConfigBuilder {
 /// to change. For more information see ticket [#285].
 ///
 /// [#285]: https://gitlab.torproject.org/tpo/core/arti/-/issues/285
-#[derive(Clone, Debug, Eq, PartialEq)]
+#[derive(Clone, Debug, Eq, PartialEq, AsRef)]
 pub struct TorClientConfig {
     /// Information about the Tor network we want to connect to.
     tor_network: dir::NetworkConfig,
@@ -312,15 +252,18 @@ pub struct TorClientConfig {
 
     /// Facility to override network parameters from the values set in the
     /// consensus.
-    override_net_params: HashMap<String, i32>,
+    override_net_params: tor_netdoc::doc::netstatus::NetParams<i32>,
 
     /// Information about how to build paths through the network.
+    #[as_ref]
     path_rules: circ::PathConfig,
 
     /// Information about preemptive circuits.
+    #[as_ref]
     preemptive_circuits: circ::PreemptiveCircuitConfig,
 
     /// Information about how to retry and expire circuits and request for circuits.
+    #[as_ref]
     circuit_timing: circ::CircuitTiming,
 
     /// Rules about which addresses the client is willing to connect to.
@@ -328,9 +271,14 @@ pub struct TorClientConfig {
 
     /// Information about timing out client requests.
     pub(crate) stream_timeouts: StreamTimeoutConfig,
+}
 
-    /// Information about system resources
-    pub system: SystemConfig,
+impl tor_circmgr::CircMgrConfig for TorClientConfig {}
+
+impl AsRef<tor_guardmgr::fallback::FallbackList> for TorClientConfig {
+    fn as_ref(&self) -> &tor_guardmgr::fallback::FallbackList {
+        self.tor_network.fallback_caches()
+    }
 }
 
 impl Default for TorClientConfig {
@@ -346,27 +294,20 @@ impl TorClientConfig {
     pub fn builder() -> TorClientConfigBuilder {
         TorClientConfigBuilder::default()
     }
+}
 
-    /// Build a DirMgrConfig from this configuration.
-    pub(crate) fn get_dirmgr_config(&self) -> Result<dir::DirMgrConfig, ConfigBuildError> {
-        let mut dircfg = dir::DirMgrConfigBuilder::default();
-        dircfg.network_config(self.tor_network.clone());
-        dircfg.schedule_config(self.download_schedule.clone());
-        dircfg.cache_path(self.storage.expand_cache_dir()?);
-        for (k, v) in &self.override_net_params {
-            dircfg.override_net_param(k.clone(), *v);
-        }
-        dircfg.build()
-    }
+impl TryInto<dir::DirMgrConfig> for &TorClientConfig {
+    type Error = ConfigBuildError;
 
-    /// Return a [`CircMgrConfig`](circ::CircMgrConfig) object based on the user's selected
-    /// configuration.
-    pub(crate) fn get_circmgr_config(&self) -> Result<circ::CircMgrConfig, ConfigBuildError> {
-        let mut builder = circ::CircMgrConfigBuilder::default();
-        builder
-            .path_rules(self.path_rules.clone())
-            .circuit_timing(self.circuit_timing.clone())
-            .build()
+    #[rustfmt::skip]
+    fn try_into(self) -> Result<dir::DirMgrConfig, ConfigBuildError> {
+        Ok(dir::DirMgrConfig {
+            network_config:      self.tor_network        .clone(),
+            schedule_config:     self.download_schedule  .clone(),
+            cache_path:          self.storage.expand_cache_dir()?,
+            override_net_params: self.override_net_params.clone(),
+            extensions:          Default::default(),
+        })
     }
 }
 
@@ -374,28 +315,35 @@ impl TorClientConfig {
 ///
 /// Unlike other builder types in Arti, this builder works by exposing an
 /// inner builder for each section in the [`TorClientConfig`].
-#[derive(Clone, Default)]
+#[derive(Clone, Default, Deserialize)]
 pub struct TorClientConfigBuilder {
     /// Inner builder for the `tor_network` section.
+    #[serde(default)]
     tor_network: dir::NetworkConfigBuilder,
     /// Inner builder for the `storage` section.
+    #[serde(default)]
     storage: StorageConfigBuilder,
     /// Inner builder for the `download_schedule` section.
+    #[serde(default)]
     download_schedule: dir::DownloadScheduleConfigBuilder,
     /// Inner builder for the `override_net_params` section.
+    #[serde(default)]
     override_net_params: HashMap<String, i32>,
     /// Inner builder for the `path_rules` section.
+    #[serde(default)]
     path_rules: circ::PathConfigBuilder,
     /// Inner builder for the `circuit_timing` section.
+    #[serde(default)]
     preemptive_circuits: circ::PreemptiveCircuitConfigBuilder,
     /// Inner builder for the `circuit_timing` section.
+    #[serde(default)]
     circuit_timing: circ::CircuitTimingBuilder,
     /// Inner builder for the `address_filter` section.
+    #[serde(default)]
     address_filter: ClientAddrConfigBuilder,
     /// Inner builder for the `stream_timeouts` section.
+    #[serde(default)]
     stream_timeouts: StreamTimeoutConfigBuilder,
-    /// Inner builder for the `system` section.
-    system: SystemConfigBuilder,
 }
 
 impl TorClientConfigBuilder {
@@ -410,7 +358,11 @@ impl TorClientConfigBuilder {
             .download_schedule
             .build()
             .map_err(|e| e.within("download_schedule"))?;
-        let override_net_params = self.override_net_params.clone();
+
+        let mut override_net_params = tor_netdoc::doc::netstatus::NetParams::new();
+        for (k, v) in &self.override_net_params {
+            override_net_params.set(k.clone(), *v);
+        }
         let path_rules = self
             .path_rules
             .build()
@@ -431,7 +383,6 @@ impl TorClientConfigBuilder {
             .stream_timeouts
             .build()
             .map_err(|e| e.within("stream_timeouts"))?;
-        let system = self.system.build().map_err(|e| e.within("system"))?;
 
         Ok(TorClientConfig {
             tor_network,
@@ -443,7 +394,6 @@ impl TorClientConfigBuilder {
             circuit_timing,
             address_filter,
             stream_timeouts,
-            system,
         })
     }
 
@@ -547,43 +497,6 @@ impl TorClientConfigBuilder {
     pub fn address_filter(&mut self) -> &mut ClientAddrConfigBuilder {
         &mut self.address_filter
     }
-
-    /// Return a mutable reference to a [`SystemConfigBuilder`].
-    ///
-    /// This section is used to configure the system resources used by Arti.
-    pub fn system(&mut self) -> &mut SystemConfigBuilder {
-        &mut self.system
-    }
-}
-
-impl From<TorClientConfig> for TorClientConfigBuilder {
-    fn from(cfg: TorClientConfig) -> TorClientConfigBuilder {
-        let TorClientConfig {
-            tor_network,
-            storage,
-            download_schedule,
-            override_net_params,
-            path_rules,
-            preemptive_circuits,
-            circuit_timing,
-            address_filter,
-            stream_timeouts,
-            system,
-        } = cfg;
-
-        TorClientConfigBuilder {
-            tor_network: tor_network.into(),
-            storage: storage.into(),
-            download_schedule: download_schedule.into(),
-            override_net_params,
-            path_rules: path_rules.into(),
-            preemptive_circuits: preemptive_circuits.into(),
-            circuit_timing: circuit_timing.into(),
-            address_filter: address_filter.into(),
-            stream_timeouts: stream_timeouts.into(),
-            system: system.into(),
-        }
-    }
 }
 
 #[cfg(test)]
@@ -594,7 +507,7 @@ mod test {
     #[test]
     fn defaults() {
         let dflt = TorClientConfig::default();
-        let b2 = TorClientConfigBuilder::from(dflt.clone());
+        let b2 = TorClientConfigBuilder::default();
         let dflt2 = b2.build().unwrap();
         assert_eq!(&dflt, &dflt2);
     }
@@ -640,11 +553,6 @@ mod test {
 
         let val = bld.build().unwrap();
 
-        // Reconstruct, rebuild, and validate.
-        let bld2 = TorClientConfigBuilder::from(val.clone());
-        let val2 = bld2.build().unwrap();
-        assert_eq!(val, val2);
-
         assert_ne!(val, TorClientConfig::default());
     }
 }
